@@ -98,11 +98,11 @@ clarabel <- function(A, b, q, P = NULL, cones, control = list(),
   n_variables <- ncol(A)
   n_constraints <- nrow(A)
   
-  if (m != n_constraints) stop("A and b incompatible dimensions.")
-  if (n != n_variables) stop("A and q incompatible dimensions.")
+  if (m != n_constraints) cli::cli_abort("{.arg A} and {.arg b} have incompatible dimensions.")
+  if (n != n_variables) cli::cli_abort("{.arg A} and {.arg q} have incompatible dimensions.")
   if (!is.null(P)) {
-    if (n != ncol(P)) stop("P and q incompatible dimensions.")
-    if (ncol(P) != nrow(P)) stop("P not square.")
+    if (n != ncol(P)) cli::cli_abort("{.arg P} and {.arg q} have incompatible dimensions.")
+    if (ncol(P) != nrow(P)) cli::cli_abort("{.arg P} is not square.")
   }
 
   # Sanitize control parameters
@@ -114,7 +114,7 @@ clarabel <- function(A, b, q, P = NULL, cones, control = list(),
   } else {
     nvars <- nvars(cones)
   }
-  if (sum(nvars) != m) stop("Constraint dimensions inconsistent with size of cones.")
+  if (sum(nvars) != m) cli::cli_abort("Constraint dimensions inconsistent with size of {.arg cones}.")
   
   ## TBD check box cone parameters, bsize > 0  & bl, bu have lengths bsize - 1
 
@@ -198,7 +198,13 @@ clarabel <- function(A, b, q, P = NULL, cones, control = list(),
 #' @param chordal_decomposition_complete_dual a boolean flag indicating complete PSD dual variables after decomposition for SDPs
 #' @return a list containing the control parameters.
 #' @details
-#' Setting `input_sparse_dropzeros` to `TRUE` will disable parametric updating functionality. See documentation of ‘dropzeros’ in Rust `struct CscMatrix` for dropping structural zeros before passing to the solver. 
+#' Setting `input_sparse_dropzeros` to `TRUE` will disable parametric updating functionality. See documentation of 'dropzeros' in Rust `struct CscMatrix` for dropping structural zeros before passing to the solver.
+#' @examples
+#' # Default control parameters
+#' ctrl <- clarabel_control()
+#' ctrl$max_iter
+#' # Custom tolerances and quiet output
+#' ctrl <- clarabel_control(verbose = FALSE, tol_gap_rel = 1e-7, max_iter = 100L)
 #' @export clarabel_control
 clarabel_control <- function(
                              ## Main algorithm settings
@@ -275,8 +281,10 @@ clarabel_control <- function(
 
   string_params <- c("direct_solve_method", "chordal_decomposition_merge_method") # Might need to uncomment character coercion below, if length > 1
   
-  if (any(sapply(params, length) != 1L)) stop("clarabel_control: arguments should be scalars!")
-  if (any(unlist(params[int_params]) < 0)) stop("clarabel_control: integer arguments should be >= 0!")
+  non_scalar <- names(which(sapply(params, length) != 1L))
+  if (length(non_scalar) > 0L) cli::cli_abort("All {.fn clarabel_control} arguments must be scalars, but {.arg {non_scalar}} {?is/are} not.")
+  neg_int <- names(which(unlist(params[int_params]) < 0))
+  if (length(neg_int) > 0L) cli::cli_abort("Integer arguments must be >= 0, but {.arg {neg_int}} {?is/are} negative.")
  
   ## The rest
   float_params <- setdiff(names(params), c(bool_params, int_params, string_params))
@@ -319,17 +327,24 @@ solver_status_descriptions <- function() {
 }
 
 ### Sanitize cone specifications
-### @param cone_spec a list of cone specifications
+### @param cone_spec a list of cone specifications, empty list or `NULL` accepted
 ### @return a named list of sanitized cone specifications and the number of variables for each cone (`nvars`)
 sanitize_cone_spec <- function(cone_spec) {
   cone_names <- names(cone_spec)
   
   ## Simple sanity checks
   if ((nc <- length(cone_names)) == 0L) {
-    stop("sanitize_cone_spec: no cone parameters specified")    
+    return(list(cones = cone_spec, nvars = c()))
+    #stop("sanitize_cone_spec: no cone parameters specified")    
   } 
-  if (length(intersect(cone_names, c("z", "l", "q", "s", "ep", "p"))) != nc) {
-    stop("sanitize_cone_spec: repeated cone parameters or unknown cone parameters specified")
+  valid_cones <- c("z", "l", "q", "s", "ep", "p")
+  unknown <- setdiff(cone_names, valid_cones)
+  if (length(unknown) > 0L || length(cone_names) != length(unique(cone_names))) {
+    if (length(unknown) > 0L) {
+      cli::cli_abort("Unknown cone parameter{?s}: {.val {unknown}}. Valid names are {.val {valid_cones}}.")
+    } else {
+      cli::cli_abort("Repeated cone parameter{?s} detected. Use {.arg strict_cone_order = FALSE} for repeated cones.")
+    }
   }
   
   ## Check lengths as noted cone parameters table for ?clarabel
@@ -338,26 +353,26 @@ sanitize_cone_spec <- function(cone_spec) {
   l <- as.integer(cone_spec[["l"]]); ll <- length(l); nvar_l <- sum(l);
   ep <- as.integer(cone_spec[["ep"]]); epl <- length(ep); nvar_ep <- sum(ep) * 3 ## 3 variables per exp cone
   if (zl > 1 || ll > 1 || epl > 1) {
-    stop("sanitize_cone_spec: z, l, ep should be scalars")
+    cli::cli_abort("Cone parameters {.val z}, {.val l}, and {.val ep} must be scalars.")
   }
   if (any(c(z, l, ep) < 0L)) {
-    stop("sanitize_cone_spec: z, l, ep should be scalars > 0")
+    cli::cli_abort("Cone parameters {.val z}, {.val l}, and {.val ep} must be non-negative.")
   }
   
   ## Now the others
   ## SOC 
   q <- as.integer(cone_spec[["q"]]); ql <- length(q); nvar_q <- sum(q);
-  if (any(q <= 0L)) stop("sanitize_cone_spec: SOC dimensions should be > 0")
+  if (any(q <= 0L)) cli::cli_abort("Second-order cone dimensions must be > 0.")
   q <- as.list(q); names(q) <- rep("q", ql);
   
   ## PSD 
   s <- as.integer(cone_spec[["s"]]); sl <- length(s); nvar_s <- if (sl > 0) sum(sapply(s, triangular_number)) else 0; ## triangular number
-  if (any(s <= 0L)) stop("sanitize_cone_spec: PSD dimensions should be > 0")
+  if (any(s <= 0L)) cli::cli_abort("PSD cone dimensions must be > 0.")
   s <- as.list(s); names(s) <- rep("s", sl);
   
   ## Power Cone
   p <- as.numeric(cone_spec[["p"]]); pl <- length(p); nvar_p <- pl * 3; ## 3 variables per power cone
-  if (any(p <= 0)) stop("sanitize_cone_spec: Power cone parameter should be > 0")
+  if (any(p <= 0)) cli::cli_abort("Power cone parameters must be > 0.")
   p <- as.list(p); names(p) <- rep("p", pl);  
 
   ## Power Cone
@@ -426,22 +441,201 @@ sanitize_gp_params_and_get_nvars <- function(gp) {
   if (length(gp) == 0) {
     list(sanitized_gp_params = list(), nvar = 0L)
   } else  {
-    sanitized_gp_params <- 
+    sanitized_gp_params <-
       lapply(gp, function(x) {
         par_names <- sort(names(x))
         if (length(x) != 2L || !identical(par_names, c("a", "n"))) {
-          stop("Generalized power cone param list should be a list of two elements named 'a' and 'n'")
+          cli::cli_abort("Generalized power cone: each entry must be a list with elements {.val a} and {.val n}.")
         }
         exps <- x[["a"]]
         if (length(exps) < 2L || any(exps <= 0) || any(exps >= 1) || abs(sum(exps) - 1.0) > 0.0) {
-          stop("Improper Generalized power cone exponents!")
+          cli::cli_abort("Generalized power cone: exponents must be in (0, 1) and sum to 1.")
         }
         n <- x[["n"]]
-        if (length(n) != 1L || n <= 0) stop("Improper Generalized power cone dimension!")
+        if (length(n) != 1L || n <= 0) cli::cli_abort("Generalized power cone: dimension {.val n} must be a positive scalar.")
         list(a = as.numeric(exps), n = as.integer(n))
       })
     list(sanitized_gp_params = sanitized_gp_params, nvar = sum(sapply(sanitized_gp_params, function(x) length(x[["a"]]) + x[["n"]])))
   }
 }
 
+# ===========================================================================
+# Persistent Solver API for Warm Starts
+# ===========================================================================
+
+#' Create a persistent Clarabel solver object
+#'
+#' @description
+#' Creates a persistent solver that can be reused across multiple
+#' solves with updated problem data (warm starts). This avoids the
+#' overhead of reallocating the solver's internal data structures when
+#' only the problem data changes but the sparsity pattern stays the
+#' same.
+#'
+#' @inheritParams clarabel
+#' @return a `ClarabelSolver` environment object with methods
+#'   `solve()`, `update_data(Px, Ax, q, b)`, and
+#'   `is_update_allowed()`
+#' @seealso [solver_solve()], [solver_update()],
+#'   [solver_is_update_allowed()], [clarabel()]
+#' @details
+#' For data updates to work, the solver settings must have
+#' `presolve_enable = FALSE`, `chordal_decomposition_enable = FALSE`,
+#' and `input_sparse_dropzeros = FALSE`. Use
+#' [solver_is_update_allowed()] to check after construction.
+#'
+#' @examples
+#' \dontrun{
+#' P <- Matrix::sparseMatrix(i = 1:2, j = 1:2, x = c(2, 1), dims = c(2, 2))
+#' A <- matrix(c(1, 0, 0, 1), nrow = 2)
+#' b <- c(1, 1)
+#' q <- c(-2, -3)
+#' cones <- list(l = 2L)
+#' ctrl <- clarabel_control(presolve_enable = FALSE, verbose = FALSE)
+#' s <- clarabel_solver(A, b, q, P, cones, control = ctrl)
+#' sol1 <- solver_solve(s)
+#' solver_update(s, q = c(-4, -1))
+#' sol2 <- solver_solve(s)
+#' }
+#' @export
+clarabel_solver <- function(A, b, q, P = NULL, cones, control = list(),
+                            strict_cone_order = TRUE) {
+
+  m <- length(b); n <- length(q)
+  n_variables <- ncol(A)
+  n_constraints <- nrow(A)
+
+  if (m != n_constraints) cli::cli_abort("{.arg A} and {.arg b} have incompatible dimensions.")
+  if (n != n_variables) cli::cli_abort("{.arg A} and {.arg q} have incompatible dimensions.")
+  if (!is.null(P)) {
+    if (n != ncol(P)) cli::cli_abort("{.arg P} and {.arg q} have incompatible dimensions.")
+    if (ncol(P) != nrow(P)) cli::cli_abort("{.arg P} is not square.")
+  }
+
+  control <- do.call(clarabel_control, control)
+  if (strict_cone_order) {
+    cones_and_nvars <- sanitize_cone_spec(cones)
+    cones <- cones_and_nvars[["cones"]]
+    nvars <- cones_and_nvars[["nvars"]]
+  } else {
+    nvars <- nvars(cones)
+  }
+  if (sum(nvars) != m) cli::cli_abort("Constraint dimensions inconsistent with size of {.arg cones}.")
+
+  if (inherits(A, "dgCMatrix")) {
+    Ai <- A@i; Ap <- A@p; Ax <- A@x
+  } else {
+    csc <- make_csc_matrix(A)
+    Ai <- csc[["matind"]]; Ap <- csc[["matbeg"]]; Ax <- csc[["values"]]
+  }
+
+  if (!is.null(P)) {
+    if (inherits(P, "dsCMatrix")) {
+      Pi <- P@i; Pp <- P@p; Px <- P@x
+    } else {
+      csc <- make_csc_symm_matrix(P)
+      Pi <- csc[["matind"]]; Pp <- csc[["matbeg"]]; Px <- csc[["values"]]
+    }
+  } else {
+    Pi <- integer(0); Pp <- integer(0); Px <- numeric(0)
+  }
+
+  ClarabelSolver$new(n_constraints, n_variables, Ai, Ap, Ax, b, q,
+                     Pi, Pp, Px, cones, control)
+}
+
+#' Solve using a persistent Clarabel solver
+#'
+#' @param solver a `ClarabelSolver` object created by
+#'   [clarabel_solver()]
+#' @return the same named list as [clarabel()]: solution vectors
+#'   `x`, `z`, `s` and solver information
+#' @seealso [clarabel_solver()], [solver_update()]
+#' @examples
+#' \dontrun{
+#' s <- clarabel_solver(A, b, q, P, cones,
+#'                      control = clarabel_control(presolve_enable = FALSE,
+#'                                                 verbose = FALSE))
+#' sol <- solver_solve(s)
+#' sol$status
+#' }
+#' @export
+solver_solve <- function(solver) {
+  solver$solve()
+}
+
+#' Update problem data on a persistent Clarabel solver
+#'
+#' @description
+#' Update one or more of P (objective), q (linear objective), A
+#' (constraints), b (constraint RHS) on an existing solver. The
+#' sparsity pattern of P and A must remain the same as the original
+#' problem; only the nonzero values can change.
+#'
+#' @param solver a `ClarabelSolver` object created by
+#'   [clarabel_solver()]
+#' @param P new upper-triangular P matrix (same sparsity), or `NULL`
+#'   to leave unchanged
+#' @param q new linear objective vector, or `NULL` to leave unchanged
+#' @param A new constraint matrix (same sparsity), or `NULL` to leave
+#'   unchanged
+#' @param b new constraint RHS vector, or `NULL` to leave unchanged
+#' @return invisible `NULL`
+#' @seealso [clarabel_solver()], [solver_solve()]
+#' @examples
+#' \dontrun{
+#' solver_update(s, q = c(-4, -1))   # update linear objective only
+#' solver_update(s, b = c(2, 2))     # update constraint RHS only
+#' sol2 <- solver_solve(s)           # re-solve with updated data
+#' }
+#' @export
+solver_update <- function(solver, P = NULL, q = NULL, A = NULL, b = NULL) {
+  ## Extract nonzero values from sparse matrices, or pass empty vectors
+  if (!is.null(P)) {
+    if (inherits(P, "dsCMatrix")) {
+      Px <- P@x
+    } else {
+      csc <- make_csc_symm_matrix(P)
+      Px <- csc[["values"]]
+    }
+  } else {
+    Px <- numeric(0)
+  }
+
+  if (!is.null(A)) {
+    if (inherits(A, "dgCMatrix")) {
+      Ax <- A@x
+    } else {
+      csc <- make_csc_matrix(A)
+      Ax <- csc[["values"]]
+    }
+  } else {
+    Ax <- numeric(0)
+  }
+
+  q_vec <- if (!is.null(q)) as.numeric(q) else numeric(0)
+  b_vec <- if (!is.null(b)) as.numeric(b) else numeric(0)
+
+  solver$update_data(Px, Ax, q_vec, b_vec)
+}
+
+#' Check if data updates are allowed on a persistent solver
+#'
+#' @description
+#' Returns `FALSE` if presolve, chordal decomposition, or
+#' `input_sparse_dropzeros` is enabled, which prevents data updates.
+#'
+#' @param solver a `ClarabelSolver` object created by
+#'   [clarabel_solver()]
+#' @return logical scalar
+#' @seealso [clarabel_solver()], [solver_update()]
+#' @examples
+#' \dontrun{
+#' solver_is_update_allowed(s)  # TRUE if presolve and chordal decomp are off
+#' }
+#' @export
+solver_is_update_allowed <- function(solver) {
+  solver$is_update_allowed()
+}
+
 
